@@ -5,9 +5,6 @@ Notes:
       on third party providers that (if using POST) won't be sending crfs
       token back.
 """
-import logging
-logger = logging.getLogger(__name__)
-
 from functools import wraps
 
 from django.http import HttpResponseRedirect, HttpResponse, \
@@ -19,7 +16,7 @@ from django.contrib import messages
 from django.views.decorators.csrf import csrf_exempt
 
 from social_auth.backends import get_backend
-from social_auth.utils import sanitize_redirect, setting
+from social_auth.utils import sanitize_redirect, setting, log
 
 
 DEFAULT_REDIRECT = setting('SOCIAL_AUTH_LOGIN_REDIRECT_URL') or \
@@ -49,16 +46,18 @@ def dsa_view(redirect_name=None):
             try:
                 return func(request, backend, *args, **kwargs)
             except Exception, e:  # some error ocurred
+                if setting('DEBUG'):
+                    raise
                 backend_name = backend.AUTH_BACKEND.name
 
-                logger.error(unicode(e), exc_info=True,
-                             extra=dict(request=request))
+                log('error', unicode(e), exc_info=True,
+                    extra=dict(request=request))
 
                 if 'django.contrib.messages' in setting('INSTALLED_APPS'):
                     from django.contrib.messages.api import error
                     error(request, unicode(e), extra_tags=backend_name)
                 else:
-                    logger.warn('Messages framework not in place, some '+
+                    log('warn', 'Messages framework not in place, some '+
                                 'errors have not been shown to the user.')
 
                 url = setting('SOCIAL_AUTH_BACKEND_ERROR_URL', LOGIN_ERROR_URL)
@@ -122,17 +121,14 @@ def disconnect(request, backend, association_id=None):
 
 def auth_process(request, backend):
     """Authenticate using social backend"""
-    # Save any defined redirect_to value into session
-    if REDIRECT_FIELD_NAME in request.REQUEST:
-        data = request.POST if request.method == 'POST' else request.GET
-        if REDIRECT_FIELD_NAME in data:
-            # Check and sanitize a user-defined GET/POST redirect_to field
-            # value.
-            redirect = data[REDIRECT_FIELD_NAME]
-
-            if setting('SOCIAL_AUTH_SANITIZE_REDIRECTS', True):
-                redirect = sanitize_redirect(request.get_host(), redirect)
-            request.session[REDIRECT_FIELD_NAME] = redirect or DEFAULT_REDIRECT
+    # Save any defined next value into session
+    data = request.POST if request.method == 'POST' else request.GET
+    if REDIRECT_FIELD_NAME in data:
+        # Check and sanitize a user-defined GET/POST next field value
+        redirect = data[REDIRECT_FIELD_NAME]
+        if setting('SOCIAL_AUTH_SANITIZE_REDIRECTS', True):
+            redirect = sanitize_redirect(request.get_host(), redirect)
+        request.session[REDIRECT_FIELD_NAME] = redirect or DEFAULT_REDIRECT
 
     if backend.uses_redirect:
         return HttpResponseRedirect(backend.auth_url())
@@ -190,4 +186,18 @@ def auth_complete(request, backend, user=None, *args, **kwargs):
     """Complete auth process. Return authenticated user or None."""
     if user and not user.is_authenticated():
         user = None
-    return backend.auth_complete(user=user, request=request, *args, **kwargs)
+
+    name = setting('SOCIAL_AUTH_PARTIAL_PIPELINE_KEY', 'partial_pipeline')
+    if request.session.get(name):
+        data = request.session.pop(name)
+        request.session.modified = True
+        return backend.continue_pipeline(pipeline_index=data['next_index'],
+                                         user=user,
+                                         request=request,
+                                         uid=data['uid'],
+                                         details=data['details'],
+                                         is_new=data['is_new'],
+                                         response=data['response'],
+                                         *args, **kwargs)
+    else:
+        return backend.auth_complete(user=user, request=request, *args, **kwargs)
